package jalview.ws2.actions.alignment;

import java.util.List;

import jalview.datamodel.AlignmentI;
import jalview.datamodel.AlignmentOrder;
import jalview.datamodel.HiddenColumns;
import jalview.ws2.actions.api.TaskEventListener;

/**
 * A data container storing the output of multiple sequence alignment services.
 * The object is constructed by an {@link AlignmentTask} on completion and
 * passed to the handler {@link TaskEventListener#taskCompleted(TaskI, Object)}
 * method as a result.
 * 
 * @author mmwarowny
 */
public class AlignmentResult
{
  final AlignmentI aln;

  final List<AlignmentOrder> alorders;

  final HiddenColumns hidden;

  AlignmentResult(AlignmentI aln, List<AlignmentOrder> alorders,
      HiddenColumns hidden)
  {
    this.aln = aln;
    this.alorders = alorders;
    this.hidden = hidden;
  }

  public AlignmentI getAlignment()
  {
    return aln;
  }

  public List<AlignmentOrder> getAlignmentOrders()
  {
    return alorders;
  }

  public HiddenColumns getHiddenColumns()
  {
    return hidden;
  }
}
